@@ -1,4 +1,4 @@
-import os, code, sys, traceback, logging, StringIO, re, threading, urlparse, weakref
+import os, code, sys, traceback, logging, StringIO, re, threading, urlparse, weakref, math
 
 import cairo
 import pango
@@ -180,6 +180,59 @@ class Header(hippo.CanvasBox):
         
 gobject.type_register(Header)
 
+class Arrow(hippo.CanvasBox):
+    __gtype_name__ = 'BigboardArrow'
+
+    UP    = 0
+    DOWN  = 1
+    LEFT  = 2
+    RIGHT = 3
+
+    MIN_SIZE = 12
+
+    def __init__(self, arrow_type, **kwargs):
+        gobject.GObject.__init__(self, **kwargs)
+        self.__arrow_type = arrow_type
+
+        self.set_clickable(True)
+
+    def do_get_content_width_request(self):
+        return (self.MIN_SIZE, self.MIN_SIZE)
+
+    def do_get_content_height_request(self, for_width):
+        return (self.MIN_SIZE, self.MIN_SIZE)
+
+    def do_paint_below_children(self, cr, dmgbox):
+        [w, h] = self.get_allocation()
+
+        color = self.get_style().get_color('color', True)
+        hippo.cairo_set_source_rgba32(cr, color)
+
+        x1 = self.props.padding_left
+        y1 = self.props.padding_top
+        x2 = w - 1 - self.props.padding_right
+        y2 = h - 1 - self.props.padding_bottom
+
+        if self.__arrow_type == self.RIGHT:
+            cr.move_to(x1, y1)
+            cr.line_to(x1, y2)
+            cr.line_to(x2, (y1 + y2) / 2)
+        elif self.__arrow_type == self.LEFT:
+            cr.move_to(x2, y1)
+            cr.line_to(x2, y2)
+            cr.line_to(x1, (y1 + y2) / 2)
+        elif self.__arrow_type == self.UP:
+            cr.move_to(x1, y2)
+            cr.line_to(x2, y2)
+            cr.line_to((x1 + x2) / 2, y1)
+        elif self.__arrow_type == self.DOWN:
+            cr.move_to(x1, y1)
+            cr.line_to(x2, y1)
+            cr.line_to((x1 + x2) / 2, y2)
+
+        cr.close_path()
+        cr.fill()
+
 class GradientHeader(hippo.CanvasGradient):
     def __init__(self, **kwargs):
         hippo.CanvasGradient.__init__(self, 
